/******************************************************************************
  Target Script for EmbeddedArtists LPC2468 OEM Board.
  
  Set CPU to 72MHz and USB clock to 48MHz, PCLK_xyz = CCLK/2. 
  
  Init external -> 16BIT <- SDRAM (Samsung K4S561632H). The init code for
  the SDRAM was taken from the EmbeddedArtists sample "lcd_lpc2478"

  Copyright (c) 2007 Rowley Associates Limited.

  Partial Copyright (c) 2008, Michael Fischer
  Added functionality of SDRAMInit.

  This file may be distributed under the terms of the License Agreement
  provided with this software.

  THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE
  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ******************************************************************************/

function PLLInit()
{
  if (TargetInterface.peekWord(0xE01FC088) & (1 << 25)) 
  {
    /* Enable PLL, disconnected */
    TargetInterface.pokeWord(0xE01FC080, 0x00000001); /* PLLCON  */
    TargetInterface.pokeWord(0xE01FC08C, 0x000000AA); /* PLLFEED */
    TargetInterface.pokeWord(0xE01FC08C, 0x00000055); /* PLLFEED */
  }
  
  /* Disable PLL, disconnected */
  TargetInterface.pokeWord(0xE01FC080, 0x00000000);   /* PLLCON  */
  TargetInterface.pokeWord(0xE01FC08C, 0x000000AA);   /* PLLFEED */
  TargetInterface.pokeWord(0xE01FC08C, 0x00000055);   /* PLLFEED */
  
  /* Setup Clock: XTAL = 12 MHz => PLL = 288 MHz, Processor Clock = 72.0 MHz */
  
  /* Enable main OSC */
  TargetInterface.pokeWord(0xE01FC1A0, 0x00000020);   /* SCS */
  
  /* Wait until main OSC is usable */
  TargetInterface.delay(100);                         
    
  /* Select main OSC, 12MHz, as the PLL clock source */
  TargetInterface.pokeWord(0xE01FC10C, 0x00000001);   /* CLKSRCSEL */
  
  /* Set M to 12 (n+1) */
  TargetInterface.pokeWord(0xE01FC084, 0x0000000B);   /* PLLCON  */
  TargetInterface.pokeWord(0xE01FC08C, 0x000000AA);   /* PLLFEED */
  TargetInterface.pokeWord(0xE01FC08C, 0x00000055);   /* PLLFEED */
  
  /* Enable PLL, disconnected */
  TargetInterface.pokeWord(0xE01FC080, 0x00000001);   /* PLLCON  */
  TargetInterface.pokeWord(0xE01FC08C, 0x000000AA);   /* PLLFEED */
  TargetInterface.pokeWord(0xE01FC08C, 0x00000055);   /* PLLFEED */
  
  /* Wait for PLL lock */
  TargetInterface.delay(100);                         
  
  /* Set CPU clock divider to 4, (n+1) => 288 / 4 = 72 */
  TargetInterface.pokeWord(0xE01FC104, 0x00000003);   /* CCLKCFG */
  
  /* Set USB clock divider to 6, (n+1) => 288 / 6 = 48 */   
  TargetInterface.pokeWord(0xE01FC108, 0x00000005);   /* USBCLKCFG */
  
  /* PCLK_xyz = CCLK/2 */
  TargetInterface.pokeWord(0xE01FC1A8, 0xAAAAAAAA);   /* PCLKSEL0 */
  TargetInterface.pokeWord(0xE01FC1AC, 0x22AAA8AA);   /* PCLKSEL1 */
  
  /* Switch to PLL clock */
  TargetInterface.pokeWord(0xE01FC080, 0x00000003);   /* PLLCON  */
  TargetInterface.pokeWord(0xE01FC08C, 0x000000AA);   /* PLLFEED */
  TargetInterface.pokeWord(0xE01FC08C, 0x00000055);   /* PLLFEED */
  
  /* Wait for PLL lock */
  TargetInterface.delay(100);  
  
  /* Initialise memory accelerator module */
  TargetInterface.pokeWord(0xE01FC000, 0x00000000);   /* MAMCR  */
  TargetInterface.pokeWord(0xE01FC004, 0x00000003);   /* MAMTIM */
  TargetInterface.pokeWord(0xE01FC000, 0x00000002);   /* MAMCR  */
}

function SDRAMInit()
{
  /* EMC_CTRL: Disable address mirror */
  TargetInterface.pokeWord(0xFFE08000, 0x00000001);   /* EMCControl */
   
  /* Turn on EMC PCLK */
  TargetInterface.pokeWord(0xE01FC0C4, 0x00000800);   /* PCONP */
  
  TargetInterface.pokeWord(0xE002C010, 0x50000000);   /* PINSEL4 ??? */
  TargetInterface.pokeWord(0xE002C014, 0x05050555);   /* PINSEL5 */
  TargetInterface.pokeWord(0xE002C018, 0x55555555);   /* PINSEL6 */
  TargetInterface.pokeWord(0xE002C020, 0x55555555);   /* PINSEL8 */
  TargetInterface.pokeWord(0xE002C024, 0x50555555);   /* PINSEL9 */

  TargetInterface.pokeWord(0xFFE08030, 0x00000002);   /* EMCDynamictRP        */
  TargetInterface.pokeWord(0xFFE08034, 0x00000003);   /* EMCDynamictRAS       */
  TargetInterface.pokeWord(0xFFE08038, 0x00000007);   /* EMCDynamictSREX      */
  TargetInterface.pokeWord(0xFFE0803C, 0x00000002);   /* EMCDynamictAPR       */
  TargetInterface.pokeWord(0xFFE08040, 0x00000005);   /* EMCDynamictDAL       */
  TargetInterface.pokeWord(0xFFE08044, 0x00000001);   /* EMCDynamictWR        */
  TargetInterface.pokeWord(0xFFE08048, 0x00000005);   /* EMCDynamictRC        */
  TargetInterface.pokeWord(0xFFE0804C, 0x00000005);   /* EMCDynamictRC        */
  TargetInterface.pokeWord(0xFFE08050, 0x00000007);   /* EMCDynamictXSR       */
  TargetInterface.pokeWord(0xFFE08054, 0x00000001);   /* EMCDynamictRRD       */
  TargetInterface.pokeWord(0xFFE08058, 0x00000002);   /* EMCDynamictMRD       */
  TargetInterface.pokeWord(0xFFE08028, 0x00000001);   /* EMCDynamicReadConfig */

  TargetInterface.pokeWord(0xFFE08104, 0x00000303);   /* EMCDynamicRASCAS0: RAS latency 2 CCLKs, CAS latenty 2 CCLKs */

  TargetInterface.pokeWord(0xFFE08100, 0x00000680);   /* EMCDynamicConfig0: 128MB, 8Mx16, 4 banks, row=12, column=9 */
  TargetInterface.delay(100);                         /* Wait 100 ms */

  TargetInterface.pokeWord(0xFFE08020, 0x00000183);   /* EMCDynamicControl: Send command: NOP */
  TargetInterface.delay(200);                         /* Wait 200 ms */
    
  TargetInterface.pokeWord(0xFFE08020, 0x00000103);   /* EMCDynamicControl: Send command: PRECHARGE-ALL, shortest possible refresh period */
  TargetInterface.pokeWord(0xFFE08024, 0x00000002);   /* EMCDynamicRefresh: Set 2 x 16CCLKs between SDRAM refresh cycles */
  TargetInterface.delay(1);                           /* Wait 1 ms */
    
  TargetInterface.pokeWord(0xFFE08024, 0x00000020);   /* EMCDynamicRefresh: Set 32 x 16CCLKs = 512CCLK = 7.1us (for 72MHz) between SDRAM refresh cycles */
    
  TargetInterface.pokeWord(0xFFE08020, 0x00000083);   /* EMCDynamicControl: Mem clock enable, CLKOUT runs, send command: MODE */

  TargetInterface.peekWord(0xA0033000);               /* Dummy read */
  
  TargetInterface.pokeWord(0xFFE08020, 0x00000000);   /* EMCDynamicControl: Send command: NORMAL */

  TargetInterface.pokeWord(0xFFE08100, 0x00080680);   /* EMCDynamicConfig0: Enable buffer */
  TargetInterface.delay(1);                           /* Wait 1 ms */
}

function Reset()
{
  TargetInterface.resetAndStop(1);
  TargetInterface.getICEBreakerRegister(5);           /* Clear out Debug Comms Data */
}

function SRAMReset()
{
  Reset();
  TargetInterface.pokeWord(0xE01FC040, 2);  /* MEMMAP */
  PLLInit();
  SDRAMInit();
}

function FLASHReset()
{
  Reset();
  TargetInterface.pokeWord(0xE01FC040, 1);  /* MEMMAP */
}
